package com.persagy.proxy.object.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.dmp.basic.model.QueryCriteria;
import com.persagy.dmp.common.helper.SpringHelper;
import com.persagy.proxy.adm.constant.GraphCodeEnum;
import com.persagy.proxy.adm.constant.RelCodeEnum;
import com.persagy.proxy.adm.request.AdmQueryCriteria;
import com.persagy.proxy.adm.request.AdmResponse;
import com.persagy.proxy.adm.service.impl.AbstractAdmBaseServiceImpl;
import com.persagy.proxy.common.entity.InstanceUrlParam;
import com.persagy.proxy.dictionary.model.AdmDict;
import com.persagy.proxy.dictionary.model.AdmObjectType;
import com.persagy.proxy.dictionary.service.IAdmDictService;
import com.persagy.proxy.object.model.AdmEquipment;
import com.persagy.proxy.object.model.AdmSystem;
import com.persagy.proxy.object.model.AdmSystemCategory;
import com.persagy.proxy.object.service.IAdmEquipmentService;
import com.persagy.proxy.object.service.IAdmSystemService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * ADM 系统 实现类
 * @author lvxy
 * @date 2021/8/18
 */
@Service
public class AdmSystemServiceImpl extends AbstractAdmBaseServiceImpl<AdmSystem> implements IAdmSystemService {

    @Autowired
    private IAdmDictService dictService;
    @Autowired
    private IAdmEquipmentService equipmentService;

    @Override
    public AdmResponse doQuery(InstanceUrlParam context, AdmQueryCriteria request, Class<AdmSystem> clazz) {
        AdmResponse response = super.doQuery(context, request, clazz);
        // 系统需要补充分类名称属性
        ensureSpecialValues(context, (List<AdmSystem>) response.getContent());
        return response;
    }

    @Override
    public void ensureSpecialValues(InstanceUrlParam context, List<AdmSystem> vos) {
        if(CollUtil.isEmpty(vos)) {
            return;
        }
        // 查询对象类型 - 只找到这个接口，应提供一个按编码查编码名称Map的接口
        AdmDict admDict = AdmDict.builder().type(AdmSystem.OBJ_TYPE).build();
        AdmResponse dicRs = dictService.queryCategory(context, admDict, AdmObjectType.class);
        List<AdmObjectType> dicList = (List<AdmObjectType>) dicRs.getContent();
        if(CollUtil.isEmpty(dicList)) {
            return;
        }
        // 转换为编码名称Map
        Map<String, String> codeNameMap = CollUtil.fieldValueAsMap(dicList, "code", "name");
        Map<String, AdmObjectType> codeNameMaps = CollUtil.fieldValueMap(dicList, "code");
        // 给系统赋值 - 同时收集设备对象，扩展设备对象赋值
        List<AdmEquipment> equipmentList = new ArrayList<>();
        for(AdmSystem vo:vos) {
            vo.setCategoryName(codeNameMap.get(vo.getClassCode()));
            if(codeNameMaps.get(vo.getClassCode()) != null){
                AdmSystemCategory admSystemCategory = new AdmSystemCategory();
                admSystemCategory.setCategoryName(codeNameMaps.get(vo.getClassCode()).getName());
                admSystemCategory.setCode(codeNameMaps.get(vo.getClassCode()).getCode());
                admSystemCategory.setProjectI(codeNameMaps.get(vo.getClassCode()).getProjectId());
                vo.setCategoryNames(admSystemCategory);
            }
            // 收集设备对象
            if(CollUtil.isEmpty(vo.getEquipmentList())) {
                continue;
            }
            equipmentList.addAll(vo.getEquipmentList());
        }
        // 对象扩展属性赋值
        equipmentService.ensureSpecialValues(context, equipmentList);
    }

    @Override
    public void processDmpCriteria(QueryCriteria dmpRequest) {
        ObjectNode criteria = dmpRequest.getCriteria();
        if (null==criteria){
            return;
        }
        if (criteria.has("floorId")){
            // 有建筑id过滤
            ObjectMapper objectMapper = SpringHelper.getBean(ObjectMapper.class);
            ObjectNode relationFrom = objectMapper.createObjectNode();
            relationFrom.put("graphCode", GraphCodeEnum.MechInArch.name());
            relationFrom.put("relCode", RelCodeEnum.Sy2Fl.name());
            relationFrom.set("objTo",criteria.get("floorId"));
            criteria.set("relationFrom",relationFrom);
            criteria.remove("floorId");
            if (criteria.has("buildingId"))
                criteria.remove("buildingId");
            return;
        }
        if (criteria.has("buildingId")){
            // 有建筑id过滤
            ObjectMapper objectMapper = SpringHelper.getBean(ObjectMapper.class);
            ObjectNode relationFrom = objectMapper.createObjectNode();
            relationFrom.put("graphCode", GraphCodeEnum.MechInArch.name());
            relationFrom.put("relCode", RelCodeEnum.Sy2Bd.name());
            relationFrom.set("objTo",criteria.get("buildingId"));
            criteria.set("relationFrom",relationFrom);
            criteria.remove("buildingId");
        }
    }
}
